import time
import subprocess
from board import SCL, SDA
import busio
from PIL import Image, ImageDraw, ImageFont
# import Adafruit_SSD1306
import adafruit_ssd1306

# Create the I2C interface.
i2c = busio.I2C(SCL, SDA)
# Create the SSD1306 OLED class.
# The first two parameters are the pixel width and pixel height. Change these
# to the right size for your display!

disp = adafruit_ssd1306.SSD1306_I2C(128, 32, i2c)
# Clear display.
disp.fill(0)
disp.show()
# Create blank image for drawing.
# Make sure to create image with mode '1' for 1-bit color.
width = disp.width
height = disp.height
image = Image.new("1", (width, height))
# Get drawing object to draw on image.
draw = ImageDraw.Draw(image)
# Draw a black filled box to clear the image.
draw.rectangle((0, 0, width, height), outline=0, fill=0)
# Draw some shapes.
# First define some constants to allow easy resizing of shapes.
padding = -2
top = padding
bottom = height - padding
# Move left to right keeping track of the current x position for drawing shapes.
x = 0
# Load default font.
font = ImageFont.load_default()
# Alternatively load a TTF font. Make sure the .ttf font file is in the
# same directory as the python script!
# Some other nice fonts to try: http://www.dafont.com/bitmap.php
# font = ImageFont.truetype('/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf', 9)
while True:
    # Draw a black filled box to clear the image.
    draw.rectangle((0, 0, width, height), outline=0, fill=0)
    # Shell scripts for system monitoring from here:
    # https://unix.stackexchange.com/questions/119126/command-to-display-memoryusage-disk-usage-and-cpu-load
    cmd = "hostname -I | cut -d' ' -f1"
    IP = subprocess.check_output(cmd, shell=True).decode("utf-8")
    cmd = 'cut -f 1 -d " " /proc/loadavg'
    CPU = subprocess.check_output(cmd, shell=True).decode("utf-8")
    cmd = "free -m | awk 'NR==2{printf \"Mem: %s/%s MB %.2f%%\",$3,$2,$3*100/$2 }'"
    MemUsage = subprocess.check_output(cmd, shell=True).decode("utf-8")
    cmd = 'df -h | awk \'$NF=="/"{printf "Disk: %d/%d GB %s", $3,$2,$5}\''
    Disk = subprocess.check_output(cmd, shell=True).decode("utf-8")
    # Write four lines of text.
    draw.text((x, top + 0), "IP: " + IP, font=font, fill=255)
    draw.text((x, top + 8), "CPU load: " + CPU, font=font, fill=255)
    draw.text((x, top + 16), MemUsage, font=font, fill=255)
    draw.text((x, top + 25), Disk, font=font, fill=255)
    # Display image.
    disp.image(image)
    disp.show()
    time.sleep(0.1)




# TODO rconcile requiremtnes
# todo implement screen stuff
# todo alerts
# todo scan for request emails

import time
from controller import Controller
from screen import Screen

# frequency of sensor update in seconds
UPDATE_FREQ = 1


def main():

    controller = Controller()

    controller.update_controller()
    controller.update_data_records(to_console=True)

    screen = Screen()
    lines = ['T1: %, T2: %'.format(controller.T1, controller.T2), 'H1: %, H2: %'.format(controller.H1, controller.H2)]
    screen.display(lines=lines)


if __name__ == '__main__':
    x = 2
    while x > 1:
        main()
        time.sleep(UPDATE_FREQ)




class AlertFactory:

    def alert_factory(self, a_type, **kwargs):
        if 'bus_disconnect' in a_type:
            return BusAlert(a_type=a_type, **kwargs)


class Alert:
    def __init__(self, a_type, **kwargs):
        self.type = a_type
        self.info = kwargs

    def process(self):
        pass


class BusAlert(Alert):
    def __init__(self, a_type, **kwargs):
        super().__init__(a_type, **kwargs)

    def process(self):
        # todo email all on alets list
        print(self.info['sensor'] + 'disconnected!')
        super().process()
        return


class EnvAlert(Alert):
    def __init__(self, a_type, **kwargs):
        super().__init__(a_type, **kwargs)

    def process(self):
        # todo email all on alets list
        if 'temp' in self.info.keys():
            print("Temperature is: " + self.info['temp'])
        elif 'hum' in self.info.keys():
            print("Humidity is: " + self.info['temp'])
        super().process()
        return
